from __future__ import unicode_literals
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render, get_object_or_404
from django.http import HttpResponse
from django.template import loader
from django.views import generic
from .models import *
from .forms import *
from .auth import * # Own wrapper for django logging in/out
from django.forms import modelformset_factory
from django.shortcuts import render
from django.db.models import Q
import collections
import datetime
from django.db.models import Count
from django.db.models import Case
from django.db.models import When

# See https://docs.djangoproject.com/en/1.10/topics/http/views/#customizing-error-views
# They should work ONLY in production! (DEBUG=False)
handler404 = 'web.views.e404_page_not_found'
handler500 = 'web.views.e500_error'
handler403 = 'web.views.e403_permission_denied'
handler400 = 'web.views.e400_bad_request'


def index(request):
	if request.user.is_authenticated():
		return redirect(appointments)
	return redirect(login)


def e404_page_not_found(request, context = None):
	return render(request, "errors/404.html", context, status=404)

def e500_error(request, context = None):
	return render(request, "errors/500.html", context, status=500)

def e403_permission_denied(request, context = None):
	return render(request, "errors/403.html", context, status=403)

def e400_bad_request(request, context = None):
	return render(request, "errors/400.html", context, status=400)


def login(request):
	context = {
		'state' : 'initial'
	}
	if request.GET and request.GET.get('error'):
		context['state'] = request.GET.get('error')

	if request.method == "POST" and request.POST:
		state, message = do_login(request)
		if state == True:
			return redirect(appointments)
		else:
			return redirect('/login?error=' + message)

	return render(request, "login.html", context)

class NotificationCount(object):
	title = ""
	count = 0
	style = ""
	type = ''

	def __init__(self, title ="Unknown", count = 0, style = "fa fa-users text-aqua", type = 'web.views.appointments'):
		self.title = title
		self.count = count
		self.style = style
		self.type = type

def get_exceeded_visits():
	return Visit.objects.filter(datetime_end__lt =  get_today_midnight_date(), is_finished = False).order_by('datetime_begin')

def get_exceeded_visit_notifications_count():
	notification = NotificationCount(
		title = "exceeded visit time",
		count = get_exceeded_visits().count(),
		style = "fa fa-thermometer-4 text-red",
		type = 'web.views.exceeded_visits')
	return notification


def get_subjects_with_no_visit():
	return Subject.objects.annotate(my_count=Count(Case(When(visit__is_finished=False, then=1)))).filter(dead=False, resigned=False, my_count=0)

def get_subject_with_no_visit_notifications_count():
	notification = NotificationCount(
		title = "subject without visit",
		count = get_subjects_with_no_visit().count(),
	 	style = "fa fa-users text-aqua",
		type = 'web.views.subject_no_visits')
	return notification


def get_visits_without_appointments_count():
	notification = NotificationCount(
		title = "unfinished visits ",
		count = get_unfinished_visits().count(),
		style = "fa fa-user-times text-yellow",
		type = 'web.views.unfinished_visits')
	return notification

def get_unfinished_visits():
	today = get_today_midnight_date()
	#list visits that have no scheduled appointments and taking place now
	return Visit.objects.annotate(my_count=Count(Case(When(appointment__status=Appointment.APPOINTMENT_STATUS_SCHEDULED, then=1)))).filter(datetime_begin__lt = today, datetime_end__gt = today, is_finished = False, my_count=0)

def get_approaching_visits_without_appointments_count():
	notification = NotificationCount(
		title = "approaching visits ",
		count = get_approaching_visits_without_appointments().count(),
		style = "fa fa-users text-aqua",
		type = 'web.views.approaching_visits_without_appointments')
	return notification

def get_approaching_visits_without_appointments():
	today = get_today_midnight_date()
	today_plus_two_months =today+datetime.timedelta(days=62)
	return Visit.objects.annotate(my_count=Count(Case(When(appointment__status=Appointment.APPOINTMENT_STATUS_SCHEDULED, then=1)))).filter(datetime_begin__gt = today, datetime_begin__lt = today_plus_two_months, is_finished = False, my_count=0)


def get_unfinished_appointments_count():
	return NotificationCount(
		title = "unfinished appointments ",
		count = get_unfinished_appointments().count(),
		style = "fa fa-history text-yellow",
		type = 'web.views.unfinished_appointments')

def get_unfinished_appointments():
	return Appointment.objects.filter(datetime_when__lt = get_today_midnight_date(), status = Appointment.APPOINTMENT_STATUS_SCHEDULED)

def get_notifications(the_user):
	workers = Worker.objects.filter(user=the_user)
	notifications = []
	count = 0;
	if len(workers)>0:
		person = workers[0]
		if person.role == Worker.ROLE_CHOICES_SECRETARY:
			notifications.append(get_exceeded_visit_notifications_count())
			notifications.append(get_visits_without_appointments_count())
			notifications.append(get_approaching_visits_without_appointments_count())
			notifications.append(get_unfinished_appointments_count())
			notifications.append(get_subject_with_no_visit_notifications_count())
			for notification in notifications:
				count += notification.count
	return (count, notifications)

"""
Saturates response with information about logged user
"""
@login_required
def wrap_response(request, template, params):
	person, role = Worker.get_details(request.user)

	notifications = get_notifications(request.user)

	final_params = params.copy()
	final_params.update({
		'person' : person,
		'role': role,
		'notifications': notifications
	})

	return render(request, template, final_params)


def logout(request):
	state, message = do_logout(request)
	return redirect('/login?error=' + message)



def visits(request):
	visit_list = Visit.objects.order_by('-datetime_begin')
	context = {
		'visit_list': visit_list
	}

	return wrap_response(request, 'visits/index.html', context)

def exceeded_visits(request):
	context = {
		'visit_list': get_exceeded_visits()
	}
	return wrap_response(request, 'visits/index.html', context)

def unfinished_visits(request):
	context = {
		'visit_list': get_unfinished_visits()
	}

	return wrap_response(request, 'visits/index.html', context)

def approaching_visits_without_appointments(request):
	context = {
		'visit_list': get_approaching_visits_without_appointments()
	}

	return wrap_response(request, 'visits/index.html', context)

def visit_details(request, id):
	displayedVisit = get_object_or_404(Visit, id=id)
	visFinished = displayedVisit.is_finished
	vid = displayedVisit.id
	displayedSubject = displayedVisit.subject
	listOfAppointments = displayedVisit.appointment_set.all()
	canFinish=True
	for appointment in listOfAppointments:
		if appointment.status == Appointment.APPOINTMENT_STATUS_SCHEDULED:
			canFinish=False;
	vform = VisitDetailForm(instance=displayedVisit)
	sform = SubjectDetailForm(instance=displayedSubject)

	return wrap_response(request, 'visits/details.html', {'vform': vform, 'sform': sform, 'loApp': listOfAppointments, 'visFinished': visFinished,'canFinish': canFinish, 'vid': vid, 'visit': displayedVisit})

def visit_mark(request, id, as_what):
	visit = get_object_or_404(Visit, id=id)
	if as_what == 'finished':
		visit.mark_as_finished()

	return redirect(visit_details, id=id)



def visit_add(request, subject_id = -1):
	if request.method == 'POST':
		form = VisitAddForm(request.POST, request.FILES)
		if form.is_valid():
			form.save()
			return redirect(visits)
	else:
		subjects = Subject.objects.filter(id=subject_id)
		subject = None
		if len(subjects)>0:
			subject = subjects[0]
		form = VisitAddForm(initial={'subject':subject})


	return wrap_response(request, 'visits/add.html', {'form': form})


def subjects(request):
	subjects_list = Subject.objects.order_by('-last_name')
	context = {
		'subjects_list': subjects_list
	}

	return wrap_response(request, 'subjects/index.html', context)

def subject_add(request):
	if request.method == 'POST':
		form = SubjectAddForm(request.POST, request.FILES)
		if form.is_valid():
			screening_number = form.cleaned_data['screening_number']
			if screening_number == '':
				screening_number = get_new_screening_number()
			form.save()
			return redirect(subjects)
	else:
		form = SubjectAddForm()

	return wrap_response(request, 'subjects/add.html', {'form': form})

def subject_no_visits(request):
	subjects_list = get_subjects_with_no_visit().order_by('-last_name')
	context = {
		'subjects_list':  subjects_list
	}

	return wrap_response(request, 'subjects/index.html', context)

def subject_details(request, id):
	the_subject = get_object_or_404(Subject, id=id)
	form = SubjectDetailForm(instance=the_subject)
	sid = id

	return wrap_response(request, 'subjects/details.html', {'form': form, 'sid': sid})


def subject_edit(request, id):
	the_subject = get_object_or_404(Subject, id=id)
	if request.method == 'POST':
		form = SubjectEditForm(request.POST, request.FILES, instance=the_subject)
		if form.is_valid():
			form.save()
			return redirect(subjects)
	else:
		form = SubjectEditForm(instance=the_subject)
	return wrap_response(request, 'subjects/edit.html', {'form': form, 'id':id})


def subject_delete(request, id):
    the_subject = get_object_or_404(Subject, id=id)
    if request.method == 'POST':
        the_subject.delete()
        return redirect(subjects)
    else:
        form = SubjectEditForm(instance=the_subject)
    return wrap_response(request, 'subjects/delete.html', {'form': form})


def subject_mark(request, id, as_what):
	who = get_object_or_404(Subject, id=id)
	if as_what == 'dead':
		who.mark_as_dead()
	elif as_what == 'rejected':
		who.mark_as_rejected()
	return redirect(subject_edit, id=id)

def appointment_mark(request, id, as_what):
	appointment = get_object_or_404(Appointment, id=id)
	if as_what == 'finished':
		appointment.mark_as_finished()
	elif as_what == 'cancelled':
		appointment.mark_as_cancelled()
	elif as_what == 'no_show':
		appointment.mark_as_no_show()
	else:
		return e500_error(request)
	return redirect(visit_details, id=appointment.visit.id)

def subject_visit_details(request, id):
	locsubject = get_object_or_404(Subject, id=id)
	visits = locsubject.visit_set.all()
	endlist = []
	for vis in visits:
		assign = vis.appointment_set.all()
		finished = vis.is_finished
		visid = vis.id
		visit_title = vis.follow_up_title()
		visform = VisitDetailForm(instance=vis)
		endlist.append((visform,assign,finished,visid,visit_title))

	#print len(endlist)
	#print endlist[0]
	return wrap_response(request, 'subjects/visitdetails.html', {'display': endlist, "id":id})

def doctors(request):
	doctors_list = Worker.objects.order_by('-last_name')
	context = {
		'doctors_list': doctors_list
	}

	return wrap_response(request, "doctors/index.html", context)


def doctor_add(request):
	if request.method == 'POST':
		form = WorkerAddForm(request.POST, request.FILES)
		if form.is_valid():
			form.save()
			return redirect(doctors)
	else:
		form = WorkerAddForm()

	return wrap_response(request, 'doctors/add.html', {'form': form})



def doctor_edit(request, doctor_id):
	the_doctor = get_object_or_404(Worker, id=doctor_id)
	if request.method == 'POST':
		form = WorkerEditForm(request.POST, request.FILES, instance=the_doctor)
		if form.is_valid():
			form.save()
			return redirect(doctors)
	else:
		form = WorkerEditForm(instance=the_doctor)
	return wrap_response(request, 'doctors/edit.html', {'form': form})


def doctor_details(request, doctor_id):
	the_doctor = get_object_or_404(Worker, id=doctor_id)
	form = WorkerDetailForm(instance=the_doctor)

	return wrap_response(request, 'doctors/details.html', {'form': form})


def doctor_availability(request, doctor_id):
	avall = Avaibility.objects.filter(person=doctor_id)

	avmon = avall.filter(day_number=1)
	avtue = avall.filter(day_number=2)
	avwed = avall.filter(day_number=3)
	avthu = avall.filter(day_number=4)
	avfri = avall.filter(day_number=5)
	avsat = avall.filter(day_number=6)
	avsun = avall.filter(day_number=7)

	context = {
		'avmon': avmon,
		'avtue': avtue,
		'avwed': avwed,
		'avthu': avthu,
		'avfri': avfri,
		'avsat': avsat,
		'avsun': avsun,
		'id': doctor_id
	}

	return wrap_response(request, "doctors/availability_index.html", context)


def doctor_availability_delete(request, doctor_id, availability_id):
	availibility = Avaibility.objects.filter(id=availability_id)
	if len(availibility) > 0:
		availibility.delete()
	return redirect(doctoravail, id=doctor_id)


def equipment_def(request):
	equipment_list = Item.objects.order_by('-name')
	context = {
		'equipment_list': equipment_list
	}

	return wrap_response(request, "eqdef/index.html", context)


def equipment_and_rooms(request):
	return wrap_response(request, "equipment_and_rooms/index.html", {})


def mail_templates(request):
	return wrap_response(request, "mail_templates/index.html", {})


"""
#An initial draft of a function that was supposed to suggest date, room and worker for an appointment
def suggest_details(Appointment appoint):

	avaibleWorkers = Worker.objects.get()
	if appoint.appointment_type__required_worker == 'DOCTOR':
		avaibleWorkers.filter(role='DOCTOR')
	elif appoint.appointment_type__required_worker == 'NURSE':
		avaibleWorkers.filter(role__in=['DOCTOR','NURSE'])
	elif appoint.appointment_type__required_worker == 'PSYCHOLOGIST':
		avaibleWorkers.filter(role__in=['DOCTOR','PSYCHOLOGIST'])

	avaibleRooms = Room.objects.get
	requireditems = appoint.appointment_type.required_equipment.filter(is_fixed=True)
	reduce(operator.and_, (Q(equipment__contains=requireditems) for x in avaibleRooms))

"""

def get_today_midnight_date():
	today = datetime.datetime.now()
	today_midnight = datetime.datetime(today.year,today.month,today.day)
	return today_midnight

def appointments(request):
	futureDate = datetime.datetime.now() + datetime.timedelta(days=93)
	planning_list = Appointment.objects.filter(datetime_when__isnull=True, visit__datetime_begin__lt = futureDate)

	today_midnight = get_today_midnight_date()
	month_ago = today_midnight + datetime.timedelta(days=-31)
	approaching_list = Appointment.objects.filter(datetime_when__gt = today_midnight, status = Appointment.APPOINTMENT_STATUS_SCHEDULED).order_by('datetime_when')
	full_list = Appointment.objects.filter(datetime_when__gt = month_ago).order_by('datetime_when')


	for plan in planning_list:
		plan.datetime_when = plan.visit.datetime_begin

	context = {
		'planning_list': planning_list,
		'approaching_list': approaching_list,
		'full_list': full_list
	}

	return wrap_response(request, "appointments/index.html",context)

def unfinished_appointments(request):
	appointments = get_unfinished_appointments()
	context = {
		'appointment_list': appointments,
	}

	return wrap_response(request, "appointments/list.html",context)

def appointment_details(request, id):
	the_appointment = get_object_or_404(Appointment, id=id)
	form = AppointmentDetailForm(instance=the_appointment)
	return wrap_response(request, 'appointments/details.html', {'form': form})


def appointment_add(request, id):
	today = datetime.datetime.now()
	today_midnight = datetime.datetime(today.year,today.month,today.day)
	month_ago = today +datetime.timedelta(days=-31)
	full_list = Appointment.objects.filter(datetime_when__gt = month_ago).order_by('datetime_when')
	if request.method == 'POST':
		form = AppointmentAddForm(request.POST, request.FILES)
		form.fields['visit'].widget = forms.HiddenInput()
		if form.is_valid():
			form.save()
			return redirect(visit_details, id=id)
	else:
		form = AppointmentAddForm(initial={'visit': id})
		form.fields['visit'].widget = forms.HiddenInput()

	return wrap_response(request, 'appointments/add.html', {'form': form, 'visitID': id, 'full_appointment_list': full_list})

def appointment_edit(request, id):
	the_appointment = get_object_or_404(Appointment, id=id)
	if request.method == 'POST':
		form = AppointmentEditForm(request.POST, request.FILES, instance=the_appointment)
		if form.is_valid():
			form.save()

			data = form.cleaned_data
			vis = data['visit']
			visit = get_object_or_404(Visit, id=vis.id)

			return redirect(appointments)
	else:
		form = AppointmentEditForm(instance=the_appointment)
	return wrap_response(request, 'appointments/edit.html', {'form': form, 'id':id, 'status': the_appointment.status})


def appointment_edit_datetime(request, id):
	the_appointment = get_object_or_404(Appointment, id=id)
	if request.method == 'POST':
		form = AppointmentEditForm(request.POST, request.FILES, instance=the_appointment)
		if form.is_valid():
			form.save()
			return redirect(appointments)
	else:
		the_appointment.datetime_when = the_appointment.visit.datetime_begin
		form = AppointmentEditForm(instance=the_appointment)
	return wrap_response(request, 'appointments/edit.html', {'form': form})
